# ------------------------------------------------------------------------------
#  es7s/core
#  (c) 2022-2023 A. Shavykin <0.delameter@gmail.com>
# ------------------------------------------------------------------------------
from __future__ import annotations

import os
import re
import subprocess
import sys
import tempfile
from os import execvpe, environ
from pathlib import Path

import pytermor as pt

from es7s.cli._foreign import ForeignInvoker, Pager
from es7s.cli._template import TemplateCommand
from es7s.shared import get_stdout, IoProxy, set_stdout
from es7s.commons import  autogen


################################################################################
# [!]                               ACHTUNG
################################################################################
# This file is a part of my long-term experiment named "Legacy code -- How bad
# can it be?" (tentative assessment: worse than a nightmare) and is maintained
# in a special way, specifically with this rule in mind:
#
#   Updating and bug fixing should be done by adding new lines; changing the
#   existing ones is prohibited unless it's the only possible way to proceed.
#
# The history can be seen in git commit log, as usual.
#
# AN ATTEMPT TO READ AND COMPREHEND THIS OR OTHER FILES RELATED TO THE EXPERI-
# MENT CAN BE DANGEROUS FOR YOUR MENTAL HEALTH AND RESULT IN A DISAPPOINTMENT
# OR LOSING FAITH IN GLORIOUS FUTURE OF THE IT INDUSTRY. YOU HAVE BEEN WARNED
################################################################################

class SgrRenderer: pass
class SequenceSGR(pt.SequenceSGR):
    _CLASSIFIER = "️"
    _VIRTUAL = True
    def print(self):
        return self.assemble()
    @property
    def opening_str(self):
        return self.assemble()

    def assemble(self) -> str:
        if len(self._params) == 0:  # NOOP
            return ""
        return (
            self.ESC_CHARACTER
            + ("[")
            + self._assemble_params()
            + (self._interm or "")
            + (self._final or "")
        )

    def __add__(self, other):
        if isinstance(other, SequenceSGR):
            return SequenceSGR(*self.params, *other.params)
        return self.assemble() + (other if other else '')
class Spans: pass
class Span: pass
class Seqs: pass

def pytermor_210_to_18_adapter():
    global seq, fmt, autof
    seq = Seqs
    fmt = Span
    autof = lambda *sq: Span(*sq)

    Seqs.build_c256 = lambda code, bg=False: SequenceSGR.init_color_indexed(code, bg)
    SequenceSGR.print = lambda self: self.assemble()
    fmt.Format = lambda *args, **kwargs: Span.init_explicit(*args, **kwargs)
    for s in dir(pt.cv):
        if s.startswith("__"):
            continue
        if isinstance(c := getattr(pt.cv, s), pt.IColorValue):
            setattr(fmt, s.lower(), c)

Seqs = pt.SeqIndex
def pytermor_258_to_210_adapter():
    global autof, fmt, seq
    seq = Seqs
    SgrRenderer.set_up = lambda _: pt.force_ansi_rendering()
    seq.build_c256 = SequenceSGR.init_color_indexed = lambda code, bg=False: SequenceSGR(48 if bg else 38, 5, code)
    autofint = (lambda *sq, s: pt.enclose(pt.SequenceSGR(*sq), s))
    autof = lambda *sq: (lambda s: autofint(*sq, s=s))
    for sq in Seqs.__dict__:
        if isinstance(SQ := getattr(Seqs, sq), pt.SequenceSGR):
            setattr(Seqs, sq, SequenceSGR(*SQ.params))
    fmt.Format = lambda sq, hard_reset_after: lambda s: s + (pt.SeqIndex.RESET.assemble() if hard_reset_after else '')
    for f in fmt.__dict__:
         if not f.startswith('_') and isinstance((ff := getattr(fmt, f)), pt.RenderColor):
            setattr(fmt, f, lambda s: SequenceSGR(ff.to_sgr("bg" in f.lower())).assemble())

@autogen
def i1():
    @autogen
    def i1a():
        BG=pt.make_color_256(234, pt.ColorTarget.BG)
        yield f"{BG}{pt.make_color_256(247)}⡀ 2.2G"
        yield f"{BG}{pt.make_color_256(247)}⣀ 3.8G"
        yield f"{BG}{pt.make_color_256(247)}⣄ 5.4G"
        yield f"{BG}{pt.make_color_256(247)}⣤ 7.7G"
        yield f"{BG}{pt.make_color_256(247)}⣦  11G"
        yield f"{BG}{pt.make_color_256( 94)}⣦  12G"
    @autogen
    def i1b():
        BG=pt.make_color_256(234, pt.ColorTarget.BG)
        yield f"{BG}{pt.make_color_256(136)}⣶  13G"
        yield f"{BG}{pt.make_color_256(172)}⣶  13G"
        yield f"{BG}{pt.make_color_256(208)}⣷  14G"
        yield f"{BG}{pt.make_color_256(202)}⣷  15G"
        yield f"{BG}{pt.make_color_256(160)}⣿  15G"
        yield f"{(mx := pt.Style(bg='red_3')).bg.to_sgr(pt.ColorTarget.BG)}{pt.make_color_256(255)}⣿{BG}"+f"{mx.bg.to_sgr()}  17G"

    while True:
        yield i1a()
        yield i1b()

@autogen
def indicator2_():
    yield '[48;5;233;32m▉▉▉[37m▏▏▏▏▏▏▏[49m▏ 31%'
    yield '[48;5;233;32m▉▉▉▉▉▉[33m▉[37m▏▏▏[49m▏[33m 67%'
    yield '[48;5;233;32m▉▉▉▉▉▉[33m▉▉[31m▉▉[49m▏[31m100%'

@autogen
def in3():
    yield '[5;1;97;48;5;160m▕[25;22;39;49m[5;1;97;48;5;160m![25;22;39;49m[5;1;97;48;5;160m▏[25;22;39;49m[5;1;97;48;5;160m [25;22;39;49m[5;1;97;48;5;160m 5%[25;22;39;49m'
    yield '[31m▕[39m[31m![39m[31m▏[39m[1;91m↑[22;39m[31m10%[39m'
    yield '[33m▕[39m[33m▁[39m[33m▏[39m[1;93m [22;39m[33m20%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▂[39m[38;5;239m▏[39m[1;32m↑[22;39m[38;5;247m25%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▂[39m[38;5;239m▏[39m[1;32m [22;39m[38;5;247m30%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▃[39m[38;5;239m▏[39m[1;32m↑[22;39m[38;5;247m50%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▅[39m[38;5;239m▏[39m[1;32m [22;39m[38;5;247m50%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▅[39m[38;5;239m▏[39m[1;32m↑[22;39m[38;5;247m65%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▇[39m[38;5;239m▏[39m[1;32m [22;39m[38;5;247m80%[39m'
    yield '[38;5;239m▕[39m[38;5;243m▇[39m[38;5;239m▏[39m[1;32m↑[22;39m[38;5;247m85%[39m'
    yield '[38;5;239m▕[39m[38;5;243m█[39m[38;5;239m▏[39m[1;32m [22;39m[38;5;247m90%[39m'
    yield '[38;5;239m▕[39m[7;32m✔[27;39m[38;5;239m▏[39m[32mF[39m[38;5;247mULL[39m'


lprot = lambda: ' prototype '

pytermor_210_to_18_adapter()
pytermor_258_to_210_adapter()
SgrRenderer.set_up(True)

iop = get_stdout()
tmp_file = None
if iop.io.isatty():
    tmp_file = open(tempfile.mkstemp()[1], 'w')
    iop = IoProxy(get_stdout().io_params, tmp_file, sys.stdout)
print = lambda s: iop.echo(s)
set_stdout(iop)

def fmt_group(m, sq):
    g0 = m.groups()[0]
    if g0.startswith(('', '', '')):
        return g0
    if g0:
        return autof(sq)(g0)
    else:
        return autof(sq + seq.CYAN)(m.groups()[1])

def fmt_header(g2, _fmt):
    if not g2:
        return ""
    if g2 and g2.strip() in ['PANE', 'PANES', 'WINDOW', 'STATUSBAR', 'LAYOUT', 'SESSION', 'SIGNATURE']:
        return autof(seq.HI_CYAN)(g2)
    if g2 and g2.strip() in ['HEAD', 'PRFIX', 'MODIF.', 'ARROW', 'BINDING', 'KEY',]:
        return (g2)
    return _fmt(g2)


s=f"""
                                                          ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                                                                     GRP  SGN   
 ████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████████████████████▓▓███████████████████████████  1   4/3 ██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████████████████████████████████████████░░░░░░░░░█
 ██BLOCK█ELEMENTS█████████████████████████ U+2580…259F ██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██BOX█DRAWINGS████████ U+2500…257F ██▓▓███████████ ╽ █████████████ 2,3  3/2 ██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ARROWS████████████████═════════════════════════════════░░░░░░░░░═ 
 ████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████████████████████▓▓█████████   ┰   ███████████  4   2/1 ██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████ U+2190…21FF ██  ← ↑ → ↓   ↔  ↕  ↖ ↘ ↗ ↙  ⤡ ⤢   ░░░░░░░░░│ 
 ██░░░░░░░░░░░░██████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┌┬┐   ┌ ┬ ┐ ███ ─ █ │ ██╴█╶██╷█╵██▓▓███████   ┧ ╁ ┟   █████████ 5,6  3/1 ██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████████████████████████  ↤ ↥ ↦ ↧      ↨  ⇱ ↸ ⇲ ⯾        ░░░░░░░░│ 
 ██ D ░░░░░███▁███▂███▃███▄███▅███▆███▇████████ U ███▔███▀████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ├┼┤  ├ ┼ ┤  ██████████████████████▓▓█████     ┎ ┯ ┒     ███████  7   4/1 ██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██SUPPLEMENTAL██████████ ⟵   ⟶    ⟷                      ░░░░░░░│ 
 ██░░░░░░░░░░░░███████⅛███¼███⅜███½███⅝███¾███⅞██████████████⅛███½███ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ └┴┘ └ ┴ ┘   ███ ━ █ ┃ ██╸█╺██╻█╹██▓▓███    ┏ ┲  ┳  ┱ ┓    █████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ARROWS████████████████ ⟻   ⟼                           ░░░░░░░│ 
 ██░░░░░░░░░░░░██████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████████████████████▓▓██  ┮ ┍ ┢ ╆ ╈ ╅ ┪ ┑ ┭  ██══════════════ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████████████████████████ ⬱ ⇇ ⇈ ⇉ ⇶ ⇊      ⤿ ⤺  ↺ ⟲ ⥀     ░░░░░░│ 
 ██ L ░░░░░███▏███▎███▍███▌███▋███▊███▉████████ R ███▕███▐████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┏┳┓ ┍┯┑ ┎┰┒ ███ ═ █ ║ ██╾█╼██╿█╽██▓▓██╼┝ ┾ ┠ ┣ ╊ ╉ ┫ ┨ ┽ ┥╾██┵┶2└┘┌┦┞3│┛┾╀│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████@A ██ U+27F0…27FF ██    ⇄ ⇅ ⇵      ⇆   ⤾ ⤻  ↻ ⟳ ⥁    ░░░░░░░│ 
 ██░░░░░░░░░░░░███████⅛███¼███⅜███½███⅝███¾███⅞██████████████⅛███½███ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┣╋┫ ┝┿┥ ┠╂┨ ██████████████████████▓▓██  ┶ ┗ ┡ ╄ ╇ ╃ ┩ ┙ ┵  ██┭┮└││ ┧┟┘┌4┘└│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████████████████████████    ⇤ ⤒ ⇥ ⤓    ↹   ⤽ ⤼       ⤳   ░░░░░░░│ 
 ██░░░░░░░░░░░░██████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┗┻┛ ┕┷┙ ┖┸┚ ███ ╌ █ ╎ ██┄█┈██┊█┆██▓▓███    ┕ ┺  ┻  ┹ ┛    ███└1╿┐ LINE│╁┽┏│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████@B ██ U+2900…297F ██    ↰ ⬑ ↱ ⬐ ⬎            ⥆ ⥅     ░░░░░░░░│ 
 ██░░░░░░░░░░░░███████████████████████████░░█████████████▟█▙██▘█▝██▞███ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████████████████████▓▓█████     ┖ ┷ ┚     █████│╾TRANSITION┘│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████████████████████████  ↵ ↲ ⬏ ↳ ↴ ↯            ⥳ ⥲ ⥴   ░░░░░░░│ 
 ██░░ F ███████▓███▒███░███████████████ P █████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╔╦╗ ╒╤╕ ╓╥╖ ███ ╍ █ ╏ ██┅█┉██┇█┋██▓▓███████   ┞ ╀ ┦   ███████╼├└╽PRIMER╃╇╄│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████████████████████████   ⇠ ⇡ ⇢ ⇣             ⥄ ⥃ ⥂     ░░░░░░░│ 
 ██░░░░░░░░░░░░███████████¾███½███¼███████░░█████████████▜█▛██▖█▗██▚███ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╠╬╣ ╞╪╡ ╟╫╢ ██████████████████████▓▓█████████   ┸   █████████┩┡┤├─┹┺│┌┘╉7╊│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓█ ⤎ ⤌  ⤍ ⤏ ⤑              ⥱ ⥵     ░░░░│ 
 ████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╚╩╝ ╘╧╛ ╙╨╜ ███  ╱╲  █ ╭╰╮╯ █ ╳ ██▓▓███████████ ╿ ███████████┪┢5│┘┱┲6┘│╅╈╆│ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████▓▓█   ⇐ ⇑ ⇒ ⇓  ⇔   ⇕   ⇖ ⇗ ⇘ ⇙      ░░░░░░░░│ 
                                                  [aug22] ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████████████████████▓▓███████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ████MISCELLANEOUS████▓▓█   ⟸   ⟹    ⟺   ⇳                ░░░░░░░│ 
  ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯                                                        ░░░░░░░░░░░░░░░░  ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██████SYMBOLS█&██████▓▓█   ⇍   ⇏    ⇎                    ░░░░░░░░│ 
                                ║22 Is a part of es7s pack  ║░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ███████████████████░░░░███████████████████░░░░███████████████████░░░░███████████████████░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ████████ARROWS███████▓▓█   ⇦ ⇧ ⇨ ⇩      ↞ ↟ ↡ ↠ ⤀ ⤁ ⤐ ⤅  ░░░░░░│ 
 48;5;23522◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢2◤║22 © 2022—2023 A. Shavykin ║ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░██[1] twix/twix    ██[2] twix/twix C  ██[3] fused/twix   ██[4] twix/bold C  ██ ░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████▓▓█   ⇽   ⇾        ↢ ⤉ ⤈ ↣ ⤔ ⤕ ⥇ ↮  ░░░░░░│ 
 48;5;23522◢2◤GEOMETRY2◢SHAPES22◤◢◤◢◤◢◤◢◤◢◤◢◤2◢║22 <0.delameter@gmail.com> ║ ░░░░░░░░░░░░░██ ╔══════╦══════╗ ░░░░██ ╔══════╤══════╗ ░░░░██ ╔═════════════╗ ░░░░██ ╔══════╦══════╗ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ████ U+2B00…21FF ████▓▓█   ⇚ ⤊ ⇛ ⤋       ⇞ ⇟   ⤟ ⤝ ⤞ ⤠   ░░░░░│ 
 48;5;235222◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢22◤◢◤◢◤◢◤◢◤◢◤◢◤◢2◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢◤◢   ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░██ ║ HEAD ║ HEAD ║ ░░░░██ ║ HEAD │ HEAD ║ ░░░░██ ║ HEAD ┃ HEAD ║ ░░░░██ ║ HEAD ║ HEAD ║ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████▓▓█     ⟰  ⟱    ⬰ ⥉ ⥈   ⤖ ⤗ ⤘       ░░░│ 
 █████████████████████ ▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄  ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╠══════╩══════╣ ░░░░██ ╠══════╪══════╣ ░░░░██ ║━━━━━━┻━━━━━━║ ░░░░██ ╚══════╩══════╝ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██┌───────────────┐██▓▓█   ⤆  ⤇                          ░░░░░░░░│ 
 ██ U+25A0 ███████████   ■   □   ▢   ▣   ▤   ▥   ▦   ▧    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ║ CELL │ CELL ║ ░░░░██ ║ CELL │ CELL ║ ░░░░██ ║ CELL │ CELL ║ ░░░░██ ┃ CELL │ CELL ┃ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│  ⬒ ⬓ ⬔ ⬕ ⬚ ⬞  │██▓▓█   ⟽  ⟾        ⤧⤨⤩⤪⤫⤬  ⤲⤱⤭⤮⤯⤰    │ 
 ████████████ …25AF ██   ▨   ▩   ▪   ▫   ▬   ▭   ▮   ▯    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ║──────┼──────║ ░░░░██ ╟──────┼──────╢ ░░░░██ ║──────┼──────║ ░░░░██ ┠──────┼──────┨ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│   ⬖ ⬗ ⬘ ⬙ ⬝   │██▓▓█   ⤂  ⤃    ⤄                     ░░░░░░░│ 
 ██ U+25B0 ███████████   ▰   ▱   ▲   △   ▴   ▵   ▶   ▷    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ║ CELL │ CELL ║ ░░░░██ ║ CELL │ CELL ║ ░░░░██ ║ CELL │ CELL ║ ░░░░██ ┃ CELL │ CELL ┃ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│    ⬟ ⬠ ⭓ ⭔    │██▓▓█   ⇪⇫⇬⇭⇮⇯ ⇰  ⬄   ⬁ ⬀ ⬂ ⬃    ⥰ ⥺  ░░░░░│ 
 ████████████ …25BF ██   ▸   ▹   ►   ▻   ▼   ▽   ▾   ▿    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╚═════════════╝ ░░░░██ ╚══════╧══════╝ ░░░░██ ╚═════════════╝ ░░░░██ ┗━━━━━━┷━━━━━━┛ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│     ⬡ ⬢ ⬣     │██▓▓█   ⬅ ⬆ ⮕ ⬇    ⬌ ⬍ ⬉ ⬈ ⬊ ⬋   ⥻ ⥹  ░░░░░░░│ 
 ██ U+25C0 ███████████   ◀   ◁   ◂   ◃   ◄   ◅   ◆   ◇    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ███████████████████░░░░███████████████████░░░░███████████████████░░░░███████████████████░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│      ⯊ ⯋      │██▓▓█   ↩ ↶ ↪ ↷        ⤣ ⤤ ⤥ ⤦  ⥼ ⥾ ⥽ ░░░░░░│ 
 ████████████ …25CF ██   ◈   ◉   ◊   ○   ◌   ◍   ◎   ●    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██[5] bold/twix    ██[6] bold/bold C  ██[7] bold C       ██[8] bold C       ██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│     ⬥ ⬦ ⬧     │██▓▓█  ⤶ ⤵ ⤷ ⤴          ⤹⤸      ⥶ ⥷ ⥸ ░│ 
 ██ U+25D0 ███████████   ◐   ◑   ◒   ◓   ◔   ◕   ◖   ◗    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┏━━━━━━┳━━━━━━┓ ░░░░██ ┏━━━━━━┳━━━━━━┓ ░░░░██ ┏━━━━━━┳━━━━━━┓ ░░░░██ ┏━━━━━━┳━━━━━━┓ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│    ⬨ ⬩ ⬪ ⬫    │██▓▓█    ⥒ ⥖   ⥘ ⥔     ⥓ ⥗     ⥙ ⥕    ░░░│ 
 ████████████ …25DF ██   ◘   ◙   ◚   ◛   ◜   ◝   ◞   ◟    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┃ HEAD ┃ HEAD ┃ ░░░░██ ┃ HEAD ┃ HEAD ┃ ░░░░██ ┃ HEAD ┃ HEAD ┃ ░░░░██ ┃ HEAD ┃ HEAD ┃ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│   ⭘ ⬬ ⬭ ⬮ ⬯   │██▓▓█ ⥚ ⥢ ⥞   ⥠ ⥣ ⥜     ⥟ ⥤ ⥛   ⥡ ⥥ ⥝ ░│ 
 ██ U+25E0 ███████████   ◠   ◡   ◢   ◣   ◤   ◥   ◦   ◧    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ┗━━━━━━┻━━━━━━┛ ░░░░██ ┣━━━━━━╇━━━━━━┫ ░░░░██ ┡━━━━━━╇━━━━━━┩ ░░░░██ ┗━━━━━━┻━━━━━━┛ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██│  ⭑ ⭒ ⯨ ⯩ ⯪ ⯫  │██▓▓█ ⥫ ⥪ ⥊ ⥑ ⥍ ⥮ ⥯ ⥌⥏   ⥋ ⥬ ⥭ ⥎      ░░░│ 
 ████████████ …25EF ██   ◨   ◩   ◪   ◫   ◬   ◭   ◮   ◯    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ║ CELL │ CELL ║ ░░░░██ ┃ CELL │ CELL ┃ ░░░░██ │ CELL │ CELL │ ░░░░██ │ CELL │ CELL │ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  ██└───────────────┘██▓▓█        ⥩ ⥦ ⥨ ⥧ ⥐                ░░░░░░│ 
 ██ U+25F0 ███████████   ◰   ◱   ◲   ◳   ◴   ◵   ◶   ◷    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╟──────┼──────╢ ░░░░██ ┠──────┼──────┨ ░░░░██ ├──────┼──────┤ ░░░░██ │──────│──────│ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░  █████████████████████▓▓██████████████████████████████████░░░░░░░░░█ 
 ████████████ …25FF ██   ◸   ◹   ◺   ◻   ◼  ◽  ◾   ◿    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ║ CELL │ CELL ║ ░░░░██ ┃ CELL │ CELL ┃ ░░░░██ │ CELL │ CELL │ ░░░░██ │ CELL │ CELL │ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░                                                     [     ]░░░░░░░░░ 
 █████████████████████ ▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀  ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██ ╚══════╧══════╝ ░░░░██ ┗━━━━━━┷━━━━━━┛ ░░░░██ └──────┴──────┘ ░░░░██ └─────────────┘ ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░                                          
                                 $ah mothefucker ^[aug22] ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ███████████████████░░░░███████████████████░░░░███████████████████░░░░███████████████████░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██████████████████████████████████████████████████████████
                                                          ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ███████████████████░░░░███████████████████░░░░█████<N> [header][/border] [attrs]... ░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██MISCELLANEOUS█TECHNICAL██████████████████ U+2300…23FF ██ 
 ░░░░                  48;5;23527                                    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ███████████████████░░░░███████████████████░░░░███████████████████░░░░███████████████████░░░░██ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  ██████████████████████████████████████████████████████████ 
 ░░░░░░░░░░░  POWERLINE EXTRAS    48;5;235;27    U+E0A0…E0A336;48;5;235─U+E0B0…E0D548;5;235                        ░░░░                   ░░░░                   ░░░░              [aug2░░░░2] ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░  █▓▓█▓▓█▓▓█▓▓█▓▓█▓▓█▓▓▓▓████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░██ 
 ░░░░░░░░░░░                     48;5;235;2748;5;235;27                                                        ....................                   ....                   ....                   ........................................................  █()░❨❩░❪❫░⦅⦆░❲❳░⸨⸩░﹙﹚████░░⌜░⌝░⌌░⌍░░░░░⎮░⎪░⎥░⎢░⎟░⎜░░░░██ 
 █████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████  ..................................................................................................................................................  ░░░░░░░░░░░░░░░░░░░░░░████░░⌞░⌟░⌎░⌏░░░░░░░░░░░░░░░░░░░░██ 
 ██   ██   ██   █   ██   █   ██   █   █   █   ██      ██    ██    █   ██   ██   █   ██   ██   ██ ...................................................................................................................................................  █▓▓▓██▓▓▓███▓▓▓▓███▓▓▓▓████░░░░░░░░░░░⎸░⏐░⎹░░⎺░⎻░⎼░⎽░⎯░░██ 
 █████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████████ ...................................................................................................................................................  █⁽³⁾░░₍₄₎░░█（）█░█｟｠████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░██ 
                                                                                                                                   [aug22] ..............................................................................................................................................  ⌇░░░░░░░░░░░░░░░░░░░░░░█████████████10███████⎯⎯███████████ 
                                                                                                                                            ...................................................................................................................................................... ████████████████████████████████████⎲██████³⎷27███████⏞███ 
 █████████████⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣿⣯⣮⢮⠗⠎⠅⡐ ⠁ █DINGBATS/CIRCLE█                  █53;36;2⋅·⦁∙•⏺●⬤◌∘◦⚬⭘○◉◎⊙⊚⦿39;55█  ....................................................................................................... ███⎛█⏜█⏜█⏜█⎞█⎛█  ᪾᪽᪻᪼██over█⎞███⎰0⎱█⌈1⌉█⎳████████████████123██ 
 ██BRAILLE██████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ ··························.....................................................··  ███⎜▓A▓B▓C▓⎟█⎜▓▓▓▓▓▓▓▓▓▓⎟███⎱0⎰█⌊2⌋█x=1██⎧ x=2; ⎫██⎯██⏟███ 
 ████PATTERNS███@0 █@1 █@2 █@3 █@4 █@5 █@6 █@7 █@8 █@9 █@A █@B █@C █@D █@E █@F █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+22C5 █│ ⋅ │█Sm█@DOT OPERATOR ███████████████████████ ··························.....................................................··  ███⎝▓⏝▓⏝▓⏝▓⎠█⎝█ ᪾  ᪽  ᪻  ᪼ █⎠████████████████⎪ y<5; ⎪██1██████ 
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+00B7 █│ · │█Po█@MIDDLE DOT █████████████████████████ ··························.....................................................··  ████████████████████████████3⌠███⎡5 7⎤███⎨ i>9; ⎬██⎶██⏠███ 
 █ U+2800…280F █ ⠀ █ ⠁ █ ⠂ █ ⠃ █ ⠄ █ ⠅ █ ⠆ █ ⠇ █ ⠈ █ ⠉ █ ⠊ █ ⠋ █ ⠌ █ ⠍ █ ⠎ █ ⠏ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2981 █│ ⦁ │█Sm█@Z NOTATION SPOT ████████████████████ ··························.....................................................··  ██⌷█⎕█⌸█⍯█⍰█⍠█⍁█⍂█⍃█⍄█⍇█⍈████⎮███⎢6 7⎥███⎪ t=2; ⎪██2██12██ 
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2219 █│ ∙ │█Sm█@BULLET OPERATOR ████████████████████ ··························.....................................................··  ███⌺█⌻█⌼█⍞█⌹█⍍█⍔█⍓█⍌█⍐█⍗████0⌡███⎣7 7⎦███⎩ a=0; ⎭██⎵██⏡███ 
 █ U+2810…281F █ ⠐ █ ⠑ █ ⠒ █ ⠓ █ ⠔ █ ⠕ █ ⠖ █ ⠗ █ ⠘ █ ⠙ █ ⠚ █ ⠛ █ ⠜ █ ⠝ █ ⠞ █ ⠟ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2022 █│ • │█Po█@BULLET █████████████████████████████ ··························.....................................................··  ██████████████████████████████████████████████████████████ 
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+23FA █│ ⏺ │█So█@BLACK CIRCLE FOR RECORD ████████████ ··························.....................................................··                                                     [jul²³] 
 █ U+2820…282F █ ⠠ █ ⠡ █ ⠢ █ ⠣ █ ⠤ █ ⠥ █ ⠦ █ ⠧ █ ⠨ █ ⠩ █ ⠪ █ ⠫ █ ⠬ █ ⠭ █ ⠮ █ ⠯ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25CF █│ ● │█So█@BLACK CIRCLE ███████████████████████ ·················································································    
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2B24 █│ ⬤ │█So█@BLACK LARGE CIRCLE █████████████████ ·················································································  ██INDICATORS██                                             
 █ U+2830…283F █ ⠰ █ ⠱ █ ⠲ █ ⠳ █ ⠴ █ ⠵ █ ⠶ █ ⠷ █ ⠸ █ ⠹ █ ⠺ █ ⠻ █ ⠼ █ ⠽ █ ⠾ █ ⠿ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25CC █│ ◌ │█So█@DOTTED CIRCLE ██████████████████████ ·················································································  ▓▓▓▓▓▓▓▓▓▓▓▓▓▓█████████████████████████████████████████            
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2218 █│ ∘ │█Sm█@RING OPERATOR ██████████████████████ ·················································································  █████████████████████{i1()}█{i1()}████{in3()}█{in3()}██           
 █ U+2840…284F █ ⡀ █ ⡁ █ ⡂ █ ⡃ █ ⡄ █ ⡅ █ ⡆ █ ⡇ █ ⡈ █ ⡉ █ ⡊ █ ⡋ █ ⡌ █ ⡍ █ ⡎ █ ⡏ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25E6 █│ ◦ │█So█@WHITE BULLET ███████████████████████ ·················································································  ██{indicator2_()}████{i1()}█{i1()}████{in3()}█{in3()}██         
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+26AC █│ ⚬ │█So█@MEDIUM SMALL WHITE CIRCLE ██████████ ·················································································  ██▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓████{i1()}█{i1()}████{in3()}█{in3()}██         
 █ U+2850…285F █ ⡐ █ ⡑ █ ⡒ █ ⡓ █ ⡔ █ ⡕ █ ⡖ █ ⡗ █ ⡘ █ ⡙ █ ⡚ █ ⡛ █ ⡜ █ ⡝ █ ⡞ █ ⡟ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2B58 █│ ⭘ │█So█@HEAVY CIRCLE ███████████████████████ ·················································································  ██{indicator2_()}████{i1()}█{i1()}████{in3()}█{in3()}██         
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25CB █│ ○ │█So█@WHITE CIRCLE ███████████████████████ ·················································································  ██▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓████{i1()}█{i1()}████{in3()}█{in3()}██            
 █ U+2860…286F █ ⡠ █ ⡡ █ ⡢ █ ⡣ █ ⡤ █ ⡥ █ ⡦ █ ⡧ █ ⡨ █ ⡩ █ ⡪ █ ⡫ █ ⡬ █ ⡭ █ ⡮ █ ⡯ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25C9 █│ ◉ │█So█@FISHEYE ████████████████████████████ ·················································································  ██{indicator2_()}████{i1()}█{i1()}████{in3()}█{in3()}██           
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25CE █│ ◎ │█So█@BULLSEYE ███████████████████████████ ·················································································  ██▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓████▓▓▓▓▓▓█▓▓▓▓▓▓████▓▓▓▓▓▓▓█▓▓▓▓▓▓▓██                       
 █ U+2870…287F █ ⡰ █ ⡱ █ ⡲ █ ⡳ █ ⡴ █ ⡵ █ ⡶ █ ⡷ █ ⡸ █ ⡹ █ ⡺ █ ⡻ █ ⡼ █ ⡽ █ ⡾ █ ⡿ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+2299 █│ ⊙ │█Sm█@CIRCLED DOT OPERATOR ███████████████ ·················································································  ██████{lprot()}██████{lprot()}████ es7s  battery ██          
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+229A █│ ⊚ │█Sm█@CIRCLED RING OPERATOR ██████████████ ·················································································  ███████████████████████████████████████████████████████                
 █ U+2880…288F █ ⢀ █ ⢁ █ ⢂ █ ⢃ █ ⢄ █ ⢅ █ ⢆ █ ⢇ █ ⢈ █ ⢉ █ ⢊ █ ⢋ █ ⢌ █ ⢍ █ ⢎ █ ⢏ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+29BF █│ ⦿ │█Sm█@CIRCLED BULLET██████████████████████ ·················································································  ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓            
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ ·················································································                                                  [sep²³]              
 █ U+2890…289F █ ⢐ █ ⢑ █ ⢒ █ ⢓ █ ⢔ █ ⢕ █ ⢖ █ ⢗ █ ⢘ █ ⢙ █ ⢚ █ ⢛ █ ⢜ █ ⢝ █ ⢞ █ ⢟ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                                                  [     ]  ············································································
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                                                              ·················································································                 █ U+2600…27FF █        
 █ U+28A0…28AF █ ⢠ █ ⢡ █ ⢢ █ ⢣ █ ⢤ █ ⢥ █ ⢦ █ ⢧ █ ⢨ █ ⢩ █ ⢪ █ ⢫ █ ⢬ █ ⢭ █ ⢮ █ ⢯ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █DINGBATS/TRIANGLE█                   █53;36;2▲△▴▵▶▷▸▹▼▽▾▿◀◁◂◃55;39█······················································                                          
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ ·················································································               ❬ ❭ ❮ ❯ ❰ ❱ ⟨ ⟩ ⟪ ⟫        
 █ U+28B0…28BF █ ⢰ █ ⢱ █ ⢲ █ ⢳ █ ⢴ █ ⢵ █ ⢶ █ ⢷ █ ⢸ █ ⢹ █ ⢺ █ ⢻ █ ⢼ █ ⢽ █ ⢾ █ ⢿ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B2 █│ ▲ │█So█@BLACK UP-POINTING TRIANGLE █████████ ·················································································                ✩ ✪ ✫ ✬ ✭ ✮ ✯ ✯           
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B3 █│ △ │█So█@WHITE UP-POINTING TRIANGLE █████████ ·················································································                ✅ ✓ ✔ ☐ ☑ ☑ ☓             
 █ U+28C0…28CF █ ⣀ █ ⣁ █ ⣂ █ ⣃ █ ⣄ █ ⣅ █ ⣆ █ ⣇ █ ⣈ █ ⣉ █ ⣊ █ ⣋ █ ⣌ █ ⣍ █ ⣎ █ ⣏ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B4 █│ ▴ │█So█@BLACK UP-POINTING SMALL TRIANGLE ███ ·················································································              ✕ ✕ ✗ ✘ ❌ ❌ ❖               
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B5 █│ ▵ │█So█@WHITE UP-POINTING SMALL TRIANGLE ███ ·················································································             ☠ ☢ ☣ ♻ ♼ ♽ ♾                
 █ U+28D0…28DF █ ⣐ █ ⣑ █ ⣒ █ ⣓ █ ⣔ █ ⣕ █ ⣖ █ ⣗ █ ⣘ █ ⣙ █ ⣚ █ ⣛ █ ⣜ █ ⣝ █ ⣞ █ ⣟ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B6 █│ ▶ │█So█@BLACK RIGHT-POINTING TRIANGLE ██████ ·················································································            ⚙ ⚛ ⚠                         
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B7 █│ ▷ │█Sm█@WHITE RIGHT-POINTING TRIANGLE ██████ ·················································································
 █ U+28E0…28EF █ ⣠ █ ⣡ █ ⣢ █ ⣣ █ ⣤ █ ⣥ █ ⣦ █ ⣧ █ ⣨ █ ⣩ █ ⣪ █ ⣫ █ ⣬ █ ⣭ █ ⣮ █ ⣯ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B8 █│ ▸ │█So█@BLACK RIGHT-POINTING SMALL TRIANGLE█ ·················································································
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25B9 █│ ▹ │█So█@WHITE RIGHT-POINTING SMALL TRIANGLE█ ·················································································             ╔══════════════════╦═
 █ U+28F0…28FF █ ⣰ █ ⣱ █ ⣲ █ ⣳ █ ⣴ █ ⣵ █ ⣶ █ ⣷ █ ⣸ █ ⣹ █ ⣺ █ ⣻ █ ⣼ █ ⣽ █ ⣾ █ ⣿ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25BC █│ ▼ │█So█@BLACK DOWN-POINTING TRIANGLE ███████ ·················································································             ║ Active operation ║ 
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25BD █│ ▽ │█So█@WHITE DOWN-POINTING TRIANGLE ███████ ·················································································             ║━━━━━━┓╮▁╮┓  mode ║ 
 ⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿⠛⣉⣤⣶⣿⠿[aug22↑][↓may23] ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25BE █│ ▾ │█So█@BLACK DOWN-POINTING SMALL TRIANGLE █ ·················································································             ║ Data ┗╰▔╰┗━━━━━━━║─
 ███████████████████████████████████████████████████████████████████████████████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25BF █│ ▿ │█So█@WHITE DOWN-POINTING SMALL TRIANGLE █ ·················································································             ║━━━━━━┓╮╮▁╮╮ mode ║ 
 █[0]█████████████████[1]███████████████████████████████[7]█████████████████[8]█ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25C0 █│ ◀ │█So█@BLACK LEFT-POINTING TRIANGLE ███████ ·················································································             ║ Data ┗╯╯▔╯╯━━━━━━║ 
 █ ⠀ ███ ⠁ █ ⠂ █ ⠄ █ ⠈ █ ⠐ █ ⠠ █ ⡀ █ ⢀ ███ ⡿ █ ⢿ █ ⣟ █ ⣯ █ ⣷ █ ⣻ █ ⣽ █ ⣾ ███ ⣿ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25C1 █│ ◁ │█Sm█@WHITE LEFT-POINTING TRIANGLE ███████ ·················································································             ║━━━━━━┏╭╭╭▁┓ mode ║ 
 ████[2]███████████[3]█████████████████[4]█████████████████[5]███████████[6]████ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25C2 █│ ◂ │█So█@BLACK LEFT-POINTING SMALL TRIANGLE █ ·················································································             ║ Data ┗╰╰╰▔┗━━━━━━║ 
 █ ⠃ ⠅ ⠆ ⠉ █ ⠇ ⠋ ⠍ ⠎ ⠓ ⠕ ⠖ ⠙ █ ⠏ ⠗ ⠛ ⠝ ⠞ ⠧ ⠫ ⠭ ⠮ ⠳ █ ⠟ ⠯ ⠷ ⠻ ⠽ ⠾ ⡏ ⡗ █ ⠿ ⡟ ⡯ ⡷ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+25C3 █│ ◃ │█So█@WHITE LEFT-POINTING SMALL TRIANGLE █ ·················································································             ║ Active operation ║ 
 █ ⠊ ⠌ ⠑ ⠒ █ ⠚ ⠜ ⠣ ⠥ ⠦ ⠩ ⠪ ⠬ █ ⠵ ⠶ ⠹ ⠺ ⠼ ⡇ ⡋ ⡍ ⡎ ⡓ █ ⡛ ⡝ ⡞ ⡧ ⡫ ⡭ ⡮ ⡳ █ ⡻ ⡽ ⡾ ⢟ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ ·················································································             ║━━━━━━┏▁┓▁▁▁┏━┓▁┏ m 
 █ ⠔ ⠘ ⠡ ⠢ █ ⠱ ⠲ ⠴ ⠸ ⡃ ⡅ ⡆ ⡉ █ ⡕ ⡖ ⡙ ⡚ ⡜ ⡣ ⡥ ⡦ ⡩ ⡪ █ ⡵ ⡶ ⡹ ⡺ ⡼ ⢏ ⢗ ⢛ █ ⢯ ⢷ ⢻ ⢽ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                                                  [     ]   ·········································································           ║ Data ┗▔┛▔┗━┛▔┗━┛━━ 
 █ ⠤ ⠨ ⠰ ⡁ █ ⡊ ⡌ ⡑ ⡒ ⡔ ⡘ ⡡ ⡢ █ ⡬ ⡱ ⡲ ⡴ ⡸ ⢇ ⢋ ⢍ ⢎ ⢓ █ ⢝ ⢞ ⢧ ⢫ ⢭ ⢮ ⢳ ⢵ █ ⢾ ⣏ ⣗ ⣛ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                                                                     ··········································································  ║   flow direction ║ 
 █ ⡂ ⡄ ⡈ ⡐ █ ⡤ ⡨ ⡰ ⢃ ⢅ ⢆ ⢉ ⢊ █ ⢕ ⢖ ⢙ ⢚ ⢜ ⢣ ⢥ ⢦ ⢩ ⢪ █ ⢶ ⢹ ⢺ ⢼ ⣇ ⣋ ⣍ ⣎ █ ⣝ ⣞ ⣧ ⣫ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █QUOTATION█&█COMMA█MARKS█      █53;36;2"'`´‹›«»‘’“”‚‛„‟⹂❛❜❝❞❟❠55;39█·················································              ║══════════════════║ 
 █ ⡠ ⢁ ⢂ ⢄ █ ⢌ ⢑ ⢒ ⢔ ⢘ ⢡ ⢢ ⢤ █ ⢬ ⢱ ⢲ ⢴ ⢸ ⣃ ⣅ ⣆ ⣉ ⣊ █ ⣓ ⣕ ⣖ ⣙ ⣚ ⣜ ⣣ ⣥ █ ⣭ ⣮ ⣳ ⣵ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ ·················································································             ║──────╮╮╮╮╮╮ mode ║ 
 █ ⢈ ⢐ ⢠ ⣀ █ ⢨ ⢰ ⣁ ⣂ ⣄ ⣈ ⣐ ⣠ █ ⣌ ⣑ ⣒ ⣔ ⣘ ⣡ ⣢ ⣤ ⣨ ⣰ █ ⣦ ⣩ ⣪ ⣬ ⣱ ⣲ ⣴ ⣸ █ ⣶ ⣹ ⣺ ⣼ █ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+0022 █│ " │█Po█@QUOTATION MARK █████████████████████ ·················································································             ║ Data ╰╰╰╰╰╰──────║ 
 ██████████████████████████████████████████████████████████████████████████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░█████ ░░░░░░░░░░░░░░ █ U+0027 █│ ' │█Po█@APOSTROPHE █████████████████████████ ·················································································             ║══════════════════║ 
 ████»4×2s¹f¾ █»3×1s¹f⅔ █»4×1s¹f¼ █»4×1s¹f¾ █»4×1s¹f½ █»2×1s¹f½ █»4×2s²f½ █»6×2s¹f⅔ ████ ░░░░░░░░░░░░░░░░░░░░░░░░ █ U+0060 █│ ` │█Sk█@GRAVE ACCENT ███████████████████████ ·················································································             ║──────╳╮╮╭╳╭╲ mod ║ 
 █▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅▅▅▅▅▅▅│▅▅█ ░░░░░░░░░░░░░░░░░░░░░░░░░░ █ U+00B4 █│ ´ │█Sk█@ACUTE ACCENT ███████████████████████ ·················································································             ║ Data ╰╯╲╳╰╳╳─╳╲─╲║─
 █  │⡾⢟⣫⣵⡾⢟⣫⣵│⡵⣫⢞⡵⣫⢞⡵⣫│⠔⢁⠔⢁⠔⢁⠔⢁│⣫⡾⣫⡾⣫⡾⣫⡾│⣡⠞⣡⠞⣡⠞⣡⠞│⡪⡪⡪⡪⡪⡪⡪⡪│⠛⣤⠛⣤⠛⣤⠛⣤│⠿⠛⣉⣤⣶⣿⠿⠛│  █ ░░░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ ···············································································             ║══════════════════║ 
 █  │⡾⢟⣫⣵⡾⢟⣫⣵│⢞⡵⣫⢞⡵⣫⢞⡵│⠔⢁⠔⢁⠔⢁⠔⢁│⣫⡾⣫⡾⣫⡾⣫⡾│⣡⠞⣡⠞⣡⠞⣡⠞│⡪⡪⡪⡪⡪⡪⡪⡪│⠛⣤⠛⣤⠛⣤⠛⣤│⣶⣿⠿⠛⣉⣤⣶⣿│  █ ░░░░░░░░░░░░░░░░ █ U+2039 203A █├‹─›┤█@LEFT/RIGHT SINGLE ANGLE QUOT. MK █ ···············································································             ║▁▁▁▁╱▁╲╲╲▁╱▁╱▁▁ mode
 █  │⡾⢟⣫⣵⡾⢟⣫⣵│⣫⢞⡵⣫⢞⡵⣫⢞│⠔⢁⠔⢁⠔⢁⠔⢁│⣫⡾⣫⡾⣫⡾⣫⡾│⣡⠞⣡⠞⣡⠞⣡⠞│⡪⡪⡪⡪⡪⡪⡪⡪│⠛⣤⠛⣤⠛⣤⠛⣤│⣉⣤⣶⣿⠿⠛⣉⣤│  █ ░░░░░░░░░░░░░░░░ █ U+00AB 00BB █├«─»┤█@LEFT/RIGHT DOUBLE ANGLE QUOT. MK █ ···············································································             ║ Data ╲╲▔╲╲▔╳▔▔▔▔▔║─
 █  │⡾⢟⣫⣵⡾⢟⣫⣵│⡵⣫⢞⡵⣫⢞⡵⣫│⠔⢁⠔⢁⠔⢁⠔⢁│⣫⡾⣫⡾⣫⡾⣫⡾│⣡⠞⣡⠞⣡⠞⣡⠞│⡪⡪⡪⡪⡪⡪⡪⡪│⠛⣤⠛⣤⠛⣤⠛⣤│⠿⠛⣉⣤⣶⣿⠿⠛│  █ ░░░░░░░░░░░░░░░░ █ U+2018 2019 █├‘─’┤█@LEFT/RIGHT SINGLE QUOTATION MARK █ ···············································································             ╚══════════════════╩═
─█──│────────│────────│────────┼────────┴────────│────────┼────────┼────────┼──█ ░░░░░░░░░░░░░░ █ U+201C 201D █├“─”┤█@LEFT/RIGHT DOUBLE QUOTATION MARK █ ··············································································
 █  │⡒⡆⣗⣓⡒⡆⣗⣓│        │        │                 │                          │  █ ░░░░░░░░░░░░░░░░ █ U+201A 201B █├‚─‛┤█@LOW/HIGH-REVER-9 SINGLE QUOT. MK █ ··············································································
 █  │⣇⣗⡓⣆⣇⣗⡓⣆│        │                                                     │  █ ░░░░░░░░░░░░░░░░ █ U+201E 201F █├„─‟┤█@LOW/HIGH-REVER-9 DOUBLE QUOT. MK █ ··············································································
 █  │⡒⡆⣗⣓⡒⡆⣗⣓│        │                                                     │  █ ░░░░░░░░░░░░░░░░ █ U+2E42 ██████├─⹂─┤█@LOW-REVERESED-9  DOUBLE QUOT. MK █ ··············································································
 █  │⣇⣗⡓⣆⣇⣗⡓⣆│        │                                   │              aug│23█ ░░░░░░░░░░ █ U+275B 275C █├❛─❜┤█@HEAVY RGL/TURNED SINGLE COMMA MK █ ··············································································                  
 ████»8×8s⁰f⅝ █@       █@       █@       █@       █@       █@       █@       █░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░███░░░░░░░░░░░░░░  █ U+275D 275E █├❝─❞┤█@HEAVY RGL/TURNED DOUBLE COMMA MK █ ··············································································
 ███████████████████████████████████████████████████████████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░████████████████████ ░░░░░░░░░░░░░░ █ U+275F 2760 █├❟─❠┤█@LOW HEAVY SINGLE/DOUBLE COMMA MK █ ··············································································
 ▅▅▅╵▅▅▅▅▅▅▅▅╵▅▅▅▅▅▅▅▅╵▅▅▅▅▅▅▅▅ ▅▅▅▅▅▅▅▅ ▅▅▅▅▅▅▅▅ ▅▅▅▅▅▅▅▅╵▅▅░░░░░░░░▅▅▅▅▅▅ ▅▅▅▅▅▅▅▅│▅▅▅ ░░░░░░░░░░░░░░ ████████████████████████████████████████████████████████ 
                                                                                 ···························································                                                  [     ] 

 ⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤⣤░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⣤  
 ⣿⢰⠂COLORED SHAPES⠐⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒⠒░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⡆⣿  
 ⣿⢸██████████████████████████████████████████████████████████████████████████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⡇⣿  
 ⣿⢸███ ⭕ ⛔ █ 🔴 🟠 🟡 🟢 🔵 🟣 🟤 ⚫ ⚪ █ ♥️  ❤️  🧡 💛 💚 💙 💜 🤎 🖤 🤍 ███░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⡇⣿  
 ⣿⢸██████████████████████████████████████████████████████████████████████████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⡇⣿  
 ⣿⢸███ 🟥 🟧 🟨 🟩 🟦 🟪 🟫 ◼️  🔳 ◻️  🔲 ██ ♦️  🔶 🔷 🔸🔹 🔺 🔻 █░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ ⭐ ⚡ ♿ ███⡇⣿  
 ⣿⢸██████████████████████████████████████████████████████████████████████████░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⡇⣿  
 ⣿⠸⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤⠤░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⠇⣿  
 ⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛⠛░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░⠛  
                                                                        [     ]░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░   
"""
def get_group_color(m):
    if m.groups('')[0] == '0.':
        return seq.YELLOW
    grp = int(m.group(2))
    if grp == 1:
        return seq.HI_RED
    elif 2 <= grp <= 3:
        return seq.HI_YELLOW
    elif grp == 4:
        return seq.HI_MAGENTA
    elif 5 <= grp <= 6:
        return seq.HI_GREEN
    elif grp == 7:
        return seq.HI_BLUE
    return seq.HI_WHITE

fmt_caption = autof(seq.CYAN+seq.build_c256(235, True))
fmt_bg = (seq.CYAN)
fmt_bg2 = (seq.CYAN+seq.DIM)
seq_bgbg = seq.build_c256(234, True)

s = re.sub('.*?', '', s)     #  aligner
s = re.sub(r'(█)([\dA-F])(█)', lambda m: '█'+f'{seq.CYAN}{seq.INVERSED}{seq.DIM}' + (m.group(2))+f'{seq.INVERSED_OFF}{seq.BOLD_DIM_OFF}█', s)
ln = 0
def color_braile2(m: re.Match):
    global ln
    char = 0
    for p in re.split('( )', m.group()):
        if p == '█' or p == ' ':
            yield p
        else:
            char += 1
            yield (f'{p}', p)[char % 2]
    if not (ln := ln + 1) % 2:
        yield ''
s = re.sub('(?<=)(.+?)(?=)', lambda m: ''.join(color_braile2(m)), s)

# s = re.sub(r'(V|var\. )?(\[\d+\]|<\w+>)([A-Za-z.-_ /]*)([A-Z )█])', lambda m:
#     autof(seq.CYAN)(m.group(1))+
#     autof(seq.YELLOW)(m.group(2))+
#     autof(seq.CYAN)(m.group(3)+m.group(4)), s)
s = re.sub(r'(v\. )?(\[\d+\]|<\w+>)([A-Za-z0-9. /\[\]]*)', lambda m:
     autof(seq.CYAN)(m.group(1) or '')+
     autof(seq.YELLOW)(m.group(2) or '')+
     autof(seq.CYAN)(m.group(3) or ''), s)

# s = re.sub(r'([^])(\b([\d])\b[^/█)\]])',
#                lambda m: m.group(1)+autof(get_group_color(m)+seq.BOLD)(m.group(2)),
#            s)
s = re.sub(r'(GRP|SGN)', lambda m: autof(seq.DIM+seq.CYAN+seq.INVERSED)(m.group(1)), s)
s = re.sub(r'(<\S+?)(d\w+?@.+?>)', lambda m: autof(seq.BLUE)(m.group(1) or '') + autof(seq.BLUE)(m.group(2) or ''), s)
s = re.sub(r'([╆╈╅╊╄╇╃╉]+)', lambda m: fmt_group(m, seq.RED), s)
s = re.sub(r'([┲┢┪┱┺┡┩┹]+)', lambda m: fmt_group(m, seq.YELLOW), s)
s = re.sub(r'([╾╼╿╽]+)', lambda m: fmt_group(m, seq.MAGENTA), s)
s = re.sub(r'([┟┧┞┦┮┭┶┵]+)', lambda m: fmt_group(m, seq.GREEN), s)
s = re.sub(r'([┽╁╀┾]+)', lambda m: fmt_group(m, seq.BLUE), s)
s = re.sub(r'([┎┯┒┏┳┓┍┠┗┣┕┻┫┑┨┙┕┛┖┷┚┸┝┿┥╂┰╋]+)', lambda m: fmt_group(m, seq.WHITE), s)
s = re.sub(r'([\x1f\x15\x16]?[┌┬┐╒╤╕╓╥╖╔╦╗├┼┤╞╪╡╟╫╢╠╬╣└┴┘╘╧╛╙╨╜╚╩╝━═│─║┃╴╶╷╵╌┄┈╎┊┆╍┅┉╏┇┋╭╮╱╲╳╰╯]+)', lambda m: fmt_group(m, seq.WHITE), s)

#s = re.sub('▒+', lambda m: seq.RESET.print()+autof(seq.build_c256(232, False)).opening_str+(len(m.group(0))*'░') + fmt_caption.opening_str, s)
#s = re.sub('░+', lambda m: seq.RESET.print()+autof(seq.build_c256(23, False)).opening_str+(len(m.group(0))*'░') + fmt_caption.opening_str, s)

            # first usage of look-behinds in regexps in my life
s = re.sub('(?<=[^])█+', lambda m: seq.COLOR_OFF.print()+fmt_bg.opening_str+(len(m.group(0))*'░')+seq.COLOR_OFF.print(), s)
s = re.sub('(?<=[^])▓+', lambda m: seq.COLOR_OFF.print()+fmt_bg2.opening_str+(len(m.group(0))*'░')+(seq.COLOR_OFF+seq.BOLD_DIM_OFF).print(), s)
#s = re.sub('(?<=[^])▒+', lambda m: seq.COLOR_OFF.print()+fmt_bg.opening_str+(len(m.group(0))*'⢕')+seq.COLOR_OFF.print(), s)
s = re.sub('(?)(~+)', lambda m: seq.COLOR_OFF.print()+autof(seq.build_c256(236)+seq.INVERSED)(m.group(1)+(len(m.group(2))*' ')), s)


#s = re.sub(r'(?:\x1b\\\x5b[0-9;]*m)?([A-Z]{4,})', lambda m: fmt_group(m, seq.OVERLINED+seq.HI_CYAN), s)
#s = re.sub(r'(\x1b\x5b[0-9;]*m)*(?:([A-Z]{4,}|BOX|&)|([A-Z (-][A-Za-z0-9,.:()/]+|   +[◀▶▲▼]|[◀▶▲▼]  +))',
s = re.sub(r'((\x1b\x5b[0-9;]*m)*)([@»]?)([:/]?)(?:((?<=[@»])[0-9A-Zaugspf×ˣ⁺¹²³⁰⁴-⁹₀-₉⅐-⅟¼½¾ .‥…_/\[\]↔-]+)|([A-Z.-]{4,}|BOX|&))',
           lambda m: (m.group(2) or '') +
                      (("", "")[m.group(3) == '»'] if m.group(3) else "") +
                     fmt_header(m.group(4) or "", autof(seq.GREEN)) +
                     fmt_header((m.group(6) or ""), autof(seq.OVERLINED + seq.HI_CYAN)) +
                     ((""+m.group(5)+"") if m.group(5) else "")
                     ,
           s)
s = re.sub(r'\x18(.+?)\x18',  # 
          lambda m: (seq.OVERLINED_OFF.assemble() + (autof(seq.CYAN + seq.DIM + seq.INVERSED)(m.group(1))) if m.group(1) else ''),
          s)
s = re.sub(r'(░)(\x1b\x5b[0-9;]*m)*(\s?U\+)?(\s?[…,]?\s?[0-9A-F]{1,4}\s?)(#?\s?)(\s?[…,]\s?)?([0-9A-F]+\s?)?(\x1b\x5b[0-9;]*m)*(░)',
           lambda m: m.group(1) + '' + fmt.Format(seq.CYAN + seq.DIM + seq.INVERSED, hard_reset_after=True)(
               '' + (autof(92)('' + (m.group(3) or ''))) + (seq.DIM + seq.BOLD + seq.CYAN).print() + m.group(4) + (
                   autof(36)(m.group(6) or '') + (seq.CYAN).print()) + (m.group(7) or '') + autof(seq.HI_CYAN)(
                   ((len(m.group(5))-1) * '?') ) ) + seq_bgbg.print() + (' ' if len(m.group(5)) > 1 else "") + seq.RESET.print() + seq_bgbg.print() + m.group(
               8) + m.group(9) + '' , s)
s = re.sub(r'(░)(\x1b\x5b[0-9;]*m)*([┌└│])(\x1b\x5b[0-9;]*m)*(░)',
           lambda m: (m.group(1) or '') + (m.group(2) or '') + f'{seq.RESET}{seq_bgbg}' + autof(
               seq.INVERSED + seq.CYAN + seq.DIM)(m.group(3)) + (m.group(4) or '') + (m.group(5) or ''), s)


s = re.sub(r'([¼½¾ʳˡˢᵇᵈᵐᵘᵣᶠₗₛ⅛⅜⅝⅞]+)', seq.CYAN.opening_str+('\\1'), s)
#s = re.sub(r'>(@+)(<?)', lambda m: (len(m.group(1)))*'▓', s)
s = re.sub(r'░(\x1b\x5b[0-9;]*m)*@(\x1b\x5b[0-9;]*m)*░', f'░{seq.RESET}█{seq_bgbg + seq.CYAN}░', s)
# language=regexp
any_=R"(?:(?:\x1b\[[\d:;]*m)*)"
s = re.sub(fr'(?<=\S)({any_}*├{any_}*─*{any_}*)(.)({any_}*─*{any_}*)(?:(.)({any_}*─*))?({any_}*┤)(?=\S)',
           rf'{(q1:=seq.CYAN+seq.DIM)}\1{(q2:=seq.HI_WHITE+seq.BOLD)}\2{q1}\3{q2}\4{q1}\5{seq.BOLD_DIM_OFF}\6', s)

#s = re.sub(r'\x1d(\x1b.\[[0-9;]*m)?', lambda m: ''.join(m.groups(''))+seq.CYAN.print(), s)
s = re.sub(r'\x1e(.+?)\x1e', autof(seq_bgbg)('\\1'), s)                                                     #  black bg
s = re.sub(r'(?)\x1f(.+?)\x1f(?)', autof(seq.CYAN+seq.DIM)('\\1'+autof(seq.INVERSED)('\\2')+'\\3'), s)    #   cyan inversed
s = re.sub(r'\x1d(.+?)\x1d',autof(seq.build_c256(238))('\\1'), s)                                           #  gray fg
s = re.sub(r'(?:\$)(ah\s+m\w+\s*\S*)', autof(seq.build_c256(237))(' \\1'), s)
s = re.sub(r'(?:([])|())(U)(\+)([A-Fa-f\d…]+)([])', lambda m:
           f'{seq.BOLD+seq.BG_BLACK+seq.GREEN+seq.build_c256(235,True)}{(m.group(1) or "")+(m.group(2) or "")}'
           f'{seq.INVERSED}{(m.group(3) or "")}' +
           f'{seq.DIM}{m.group(4)}{seq.CYAN}{m.group(5)}'+ f'{seq.INVERSED_OFF}' + fmt_caption(m.group(6)) + f'{seq.BOLD_DIM_OFF}',
           s)

s = re.sub(r'\x15', '', s)  # 
s = re.sub(r'\x16(.+?)\x16', autof(seq.CYAN+seq.DIM)('\\1'), s)      # 
s = re.sub(r'\x19([0-9;]+)', lambda m: SequenceSGR(*(int(c) for c in m.group(1).split(';'))).assemble(), s)     # 
s = re.sub(r'\x0e([0-9;]+)(.*?)\x0f', lambda m: autof(SequenceSGR(*(int(c) for c in m.group(1).split(';'))))(m.group(2)), s)    #  
s = re.sub(r'\x17',fmt_header(' ', autof(seq.CYAN + seq.DIM + seq.INVERSED)), s)  # 

#if common.get_terminal_width() < 140:
#    print(fmt.yellow('Table example truncated -- terminal is not wide enough (need 145+)'))
#    s = re.sub(r'\x1c.+', '', s)    # 

s = re.sub('[{}]+'.format(''.join([
    '',     '', '', '',  '', '', '', '',
  #  00      01     02     03      04     05     06     07
    '',   '\t',   '',   '',   '',  '\r',  '',  '',
  #  08      09     0a \n  0b      0c     0d     0e     0f
    '',  '', '', '',  '', '', '', '',
  #  0f      11     12     13      14     15     16     17
    '',  '',  '',  '',    '',  '',  '',  '',  '',
  #  18      19     1a     1b   1c     1d     1e     1f     7f
])), '', s)
print(s)

TemplateCommand(str(Path(__file__).parent.joinpath("supsubscripts.txt_").absolute())).run()

if tmp_file:
    tmp_file.flush()
    Pager().open(tmp_file.name)
